window.Gleam.initSearch([{"doc":"xmlm","title":"xmlm","content":"# xmlm\n\nXmlm is a pull-based XML parser for Gleam, in a similar style as the OCaml [xmlm](https://erratique.ch/software/xmlm/doc/Xmlm/index.html) library.\n\n## Documentation &amp; Usage\n\n- For documentation and some usage examples, see the API docs in `src/xmlm.gleam`\n- For complete XML processing examples, see the `test/examples` directory.\n- Tests include XML spec conformance tests (currently the OASIS/NIST suite only), which also is currently serving as the &quot;documentation&quot; for where it diverges from the XML spec (i.e., in the same ways as the OCaml library on which it is based does), as well as the integration tests from this Rust library (https://github.com/RazrFalcon/xmlparser/tree/master/tests/integration).\n\n## Hacking\n\nCheck out the `justfile` for various utilities and helpers.  (You will need [just](https://just.systems/) installed to use it.)\n\n### XML Conformance Tests\n\n- XML Conformance tests are located in `test/xmlconf`. \n  - Within each subdirectory, there is a `gen.gleam` file that auto-generates the tests.\n  - These files include some rules about when tests that are expected to fail according to the spec will actually pass in this package, and vice-versa.\n- Corresponding XML test files are located in `test/test_files/xmlconf`. \n- To generate the tests, run `just gen_xmlconf_tests`.\n  - Currently, only the oasis tests are used, but more will be incorporated.\n\n### Benchmarks\n\nYou can run various benchmarks with:\n\n- `just bench_compare_erlang`\n- `just bench_compare_javascript`\n- `just bench_signals_erlang`\n- `just bench_signals_javascript`\n\nNote that the JavaScript benchmarks are fairly slow to run.\n\n### Profiling Erlang Code\n\nRun the Erlang shell with `gleam shell`.  Then input the following:\n\n```\nfprof:trace(start).\nbench@run_in_shell:main().\nfprof:trace(stop).\nfprof:profile().\nfprof:analyse({dest, &quot;_output/profile.fprof&quot;}).\n```\n\nIf you want to profile using a longer file, you can run the following instead.\n\n\n```\nInput = bench@run_in_shell:make_input(&quot;/home/ryan/projects/gleam/xmlm/test/test_files/33397721_long.xml&quot;).\nfprof:trace(start).\nbench@run_in_shell:no_op(Input).\nfprof:trace(stop).\nfprof:profile().\nfprof:analyse({dest, &quot;_output/profile.fprof&quot;}).\n```\n\n(You can also use `bench@run_in_shell:count_start_signals(Input)` rather than `no_op`.)\n\nOnce either one of the above finishes running (it shouldn&#39;t take more than a few moments), then run `just erlgrind` to view the profile with kchachegrind.\n\nNote that this requires both [kcachegrind](https://kcachegrind.sourceforge.net/html/Home.html) and [erlgrind](https://github.com/isacssouza/erlgrind) to be installed.\n\n## Roadmap\n\n- [ ] Accept input sources other than string or bit array.  (Ideally, some sort of abstraction that would also allow processing a stream of data.)\n\n## Acknowledgements\n\nVery heavily inspired by OCaml&#39;s [xmlm](https://erratique.ch/software/xmlm) package by Daniel Bünzli.\n\n## License\n\n[![license MIT or Apache\n2.0](https://img.shields.io/badge/license-MIT%20or%20Apache%202.0-blue)](https://github.com/mooreryan/gleam_qcheck)\n\nCopyright (c) 2024 Ryan M. Moore\n\nLicensed under the Apache License, Version 2.0 or the MIT license, at your option. This program may not be copied, modified, or distributed except according to those terms.","url":"index.html"},{"doc":"xmlm","title":"Attribute","content":"pub type Attribute {\n  Attribute(name: Name, value: String)\n}\n Type for attributes.\n \n ## Example\n \n In following XML fragment `&lt;fruit color=&quot;green&quot;&gt;`, the attribute \n `color=&quot;green&quot;` would look like this: \n \n \n Attribute(name: Name(uri: &quot;&quot;, local: &quot;color&quot;), value: &quot;green&quot;)\n \n \nAttribute(name: Name, value: String)\n\nname\n&lt;p&gt;The &lt;code&gt;name&lt;/code&gt; of the &lt;code&gt;Attribute&lt;/code&gt;&lt;/p&gt;\n\nvalue\n&lt;p&gt;The &lt;code&gt;value&lt;/code&gt; of the &lt;code&gt;Attribute&lt;/code&gt;&lt;/p&gt;\n\nSynonyms:\nxmlm.Attribute\nxmlm Attribute","url":"xmlm.html#Attribute"},{"doc":"xmlm","title":"Encoding","content":"pub type Encoding {\n  Utf8\n  Utf16\n  Utf16Be\n  Utf16Le\n  Iso8859x1\n  Iso8859x15\n  UsAscii\n}\n The type for character encodings\n \nUtf8\n\n\nUtf16\n UTF-16 endianness is determined from the\n [BOM](https://www.unicode.org/faq/utf_bom.html#BOM).\n \n\nUtf16Be\n UTF-16 big-endian\n \n\nUtf16Le\n UTF-16 big-endian\n \n\nIso8859x1\n\n\nIso8859x15\n\n\nUsAscii\n\n\nSynonyms:\nxmlm.Encoding\nxmlm Encoding","url":"xmlm.html#Encoding"},{"doc":"xmlm","title":"Input","content":"pub opaque type Input\n The type for input abstractions.\n \n\nSynonyms:\nxmlm.Input\nxmlm Input","url":"xmlm.html#Input"},{"doc":"xmlm","title":"InputError","content":"pub opaque type InputError\n The type of error returned by any &quot;inputing&quot; functions.\n \n\nSynonyms:\nxmlm.InputError\nxmlm InputError","url":"xmlm.html#InputError"},{"doc":"xmlm","title":"Name","content":"pub type Name {\n  Name(uri: String, local: String)\n}\n Type for names of attribute and elements.  An empty `uri` represents a\n name without a namespace, i.e., an unprefixed name that is not under the \n scope of a default namespace.\n \nName(uri: String, local: String)\n\nuri\n&lt;p&gt;The URI of the &lt;code&gt;Name&lt;/code&gt;.&lt;/p&gt;\n&lt;p&gt;Note that this likely* will not be the literal value of the prefix\nstring before the &lt;code&gt;:&lt;/code&gt;.  E.g.,&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-xml&quot;&gt;&amp;lt;a xmlns:snazzy=&quot;https://www.example.com/snazzy&quot;&amp;gt;\n  &amp;lt;snazzy:b /&amp;gt;\n&amp;lt;/a&amp;gt;\n&lt;/code&gt;&lt;/pre&gt;\n&lt;p&gt;The &lt;code&gt;b&lt;/code&gt; tag would look something like this:&lt;/p&gt;\n&lt;pre&gt;&lt;code class=&quot;language-gleam&quot;&gt;Tag(\n  name: Name(uri: &quot;https://www.example.com/snazzy&quot;, local: &quot;b&quot;), \n  attributes: []\n)\n&lt;/code&gt;&lt;/pre&gt;\n&lt;p&gt;Note how the &lt;code&gt;uri&lt;/code&gt; is not &lt;code&gt;&quot;snazzy&quot;&lt;/code&gt;, but\n&lt;code&gt;&quot;https://www.example.com/snazzy&quot;&lt;/code&gt;.&lt;/p&gt;\n&lt;p&gt;*I say “likely”, because you could define a &lt;code&gt;namespace_callback&lt;/code&gt; that\nmaps URIs to themselves rather than a URI.&lt;/p&gt;\n\nlocal\n&lt;p&gt;The non-prefixed (i.e., &lt;code&gt;local&lt;/code&gt;) part of the &lt;code&gt;Name&lt;/code&gt;.&lt;/p&gt;\n\nSynonyms:\nxmlm.Name\nxmlm Name","url":"xmlm.html#Name"},{"doc":"xmlm","title":"Signal","content":"pub type Signal {\n  Dtd(Option(String))\n  ElementStart(Tag)\n  ElementEnd\n  Data(String)\n}\n The type for signals\n \n A well-formed sequence of signals belongs to the language of the document \n grammar:\n \n \n document := Dtd tree ;\n tree     := ElementStart child ElementEnd ;\n child    := ( Data trees ) | trees ;\n trees    := ( tree child ) | epsilon ;\n \n \n Note the `trees` production which expresses the fact there there will never \n be two consecutive `Data` signals in the children of an element.\n \n The `Input` type and functions that work with it deal only with well-formed \n signal sequences, else `Errors` are returned.\n \nDtd(Option(String))\n\n\nElementStart(Tag)\n\n\nElementEnd\n\n\nData(String)\n\n\nSynonyms:\nxmlm.Signal\nxmlm Signal","url":"xmlm.html#Signal"},{"doc":"xmlm","title":"Tag","content":"pub type Tag {\n  Tag(name: Name, attributes: List(Attribute))\n}\n The type for an element tag.\n \nTag(name: Name, attributes: List(Attribute))\n\nname\n&lt;p&gt;Name of the tag&lt;/p&gt;\n\nattributes\n&lt;p&gt;Attribute list of the tag&lt;/p&gt;\n\nSynonyms:\nxmlm.Tag\nxmlm Tag","url":"xmlm.html#Tag"},{"doc":"xmlm","title":"attribute_to_string","content":"pub fn attribute_to_string(attribute: Attribute) -&gt; String\n Convert `attribute` into an unspecified string representation.\n \nSynonyms:\nxmlm.attribute_to_string\nxmlm attribute_to_string","url":"xmlm.html#attribute_to_string"},{"doc":"xmlm","title":"attributes_to_string","content":"pub fn attributes_to_string(\n  attributes: List(Attribute),\n) -&gt; String\n Convert `attributes` into an unspecified string representation.\n \nSynonyms:\nxmlm.attributes_to_string\nxmlm attributes_to_string","url":"xmlm.html#attributes_to_string"},{"doc":"xmlm","title":"document_tree","content":"pub fn document_tree(\n  input: Input,\n  element_callback element_callback: fn(Tag, List(a)) -&gt; a,\n  data_callback data_callback: fn(String) -&gt; a,\n) -&gt; Result(#(Option(String), a, Input), InputError)\n `xmlm.document_tree(input, element_callback, data_callback)` reads a \n complete, well-formed sequence of signals.\n \n See [tree](#tree) for getting a tree produced by a single `Signal`. \n\nSynonyms:\nxmlm.document_tree\nxmlm document_tree","url":"xmlm.html#document_tree"},{"doc":"xmlm","title":"eoi","content":"pub fn eoi(input: Input) -&gt; Result(#(Bool, Input), InputError)\n `eoi(input)` tells if the end of input is reached.\n \nSynonyms:\nxmlm.eoi\nxmlm eoi","url":"xmlm.html#eoi"},{"doc":"xmlm","title":"fold_signals","content":"pub fn fold_signals(\n  over input: Input,\n  from acc: a,\n  with f: fn(a, Signal) -&gt; a,\n) -&gt; Result(#(a, Input), InputError)\n `xmlm.fold_signals(over: input, from: acc, with: f)` reduces the `Signals` \n of the `input` to a single value starting with `acc` by calling the given \n function `f` on each `Signal` in the `input`.\n \nSynonyms:\nxmlm.fold_signals\nxmlm fold_signals","url":"xmlm.html#fold_signals"},{"doc":"xmlm","title":"from_bit_array","content":"pub fn from_bit_array(source: BitArray) -&gt; Input\n `xmlm.from_bit_array(source)` returns a new `Input` abstraction from the \n given `source`.\n \nSynonyms:\nxmlm.from_bit_array\nxmlm from_bit_array","url":"xmlm.html#from_bit_array"},{"doc":"xmlm","title":"from_string","content":"pub fn from_string(source: String) -&gt; Input\n `xmlm.from_string(source)` returns a new `Input` abstraction from the given\n `source`.\n \nSynonyms:\nxmlm.from_string\nxmlm from_string","url":"xmlm.html#from_string"},{"doc":"xmlm","title":"input_error_to_string","content":"pub fn input_error_to_string(input_error: InputError) -&gt; String\n Converts the `input_error` into a non-specified human readable format.\n \nSynonyms:\nxmlm.input_error_to_string\nxmlm input_error_to_string","url":"xmlm.html#input_error_to_string"},{"doc":"xmlm","title":"input_to_string","content":"pub fn input_to_string(input: Input) -&gt; String\n Convert `input` into an unspecified string representation.\n \nSynonyms:\nxmlm.input_to_string\nxmlm input_to_string","url":"xmlm.html#input_to_string"},{"doc":"xmlm","title":"name_to_string","content":"pub fn name_to_string(name: Name) -&gt; String\n Convert `name` into an unspecified string representation.\n \nSynonyms:\nxmlm.name_to_string\nxmlm name_to_string","url":"xmlm.html#name_to_string"},{"doc":"xmlm","title":"peek","content":"pub fn peek(input: Input) -&gt; Result(#(Signal, Input), InputError)\n `xmlm.peek(input)` is the same as `xmlm.signal(input)` except that\n the signal is not removed from the sequence.\n \nSynonyms:\nxmlm.peek\nxmlm peek","url":"xmlm.html#peek"},{"doc":"xmlm","title":"signal","content":"pub fn signal(\n  input: Input,\n) -&gt; Result(#(Signal, Input), InputError)\n `xmlm.signal(input)` inputs a `Signal`.  \n \n Repeatedly invoking the function with the same input abstraction will\n either generate a well-formed sequence apple shton of signals or raise an\n error. Additionally, no two consecutive Data signals can appear in the\n sequence, and their strings will always be non-empty.\n \n Note: Currently, after a well-formed sequence has been input, another \n sequence can be input.  However, this behavior is **deprecated**. (It is \n inherited from the OCaml library on which this one is based, and will chage \n at some point in the future.)\n \nSynonyms:\nxmlm.signal\nxmlm signal","url":"xmlm.html#signal"},{"doc":"xmlm","title":"signal_to_string","content":"pub fn signal_to_string(signal: Signal) -&gt; String\n Convert `signal` into an unspecified string representation.\n \nSynonyms:\nxmlm.signal_to_string\nxmlm signal_to_string","url":"xmlm.html#signal_to_string"},{"doc":"xmlm","title":"signals","content":"pub fn signals(\n  input: Input,\n) -&gt; Result(#(List(Signal), Input), InputError)\n Return a list of all `Signals` in the given `input`.\n \nSynonyms:\nxmlm.signals\nxmlm signals","url":"xmlm.html#signals"},{"doc":"xmlm","title":"signals_to_string","content":"pub fn signals_to_string(signals: List(Signal)) -&gt; String\n Convert `signals` into an unspecified string representation.\n \nSynonyms:\nxmlm.signals_to_string\nxmlm signals_to_string","url":"xmlm.html#signals_to_string"},{"doc":"xmlm","title":"tag_to_string","content":"pub fn tag_to_string(tag: Tag) -&gt; String\n Convert `tag` into an unspecified string representation.\n \nSynonyms:\nxmlm.tag_to_string\nxmlm tag_to_string","url":"xmlm.html#tag_to_string"},{"doc":"xmlm","title":"tree","content":"pub fn tree(\n  input: Input,\n  element_callback element_callback: fn(Tag, List(a)) -&gt; a,\n  data_callback data_callback: fn(String) -&gt; a,\n) -&gt; Result(#(a, Input), InputError)\n `xmlm.tree(input, element_callback, data_callback)` inputs signals in \n different ways depending on the next signal.\n \n If the next signal is a...\n \n - `Data` signal, `tree` inputs the signal and invokes `data_callback` with \n   the character data of the signal.\n - `ElementStart` signal, `tree` inputs the sequence of signals until its \n   matching `ElementEnd` and envokes `element_callback` and `data_callback` \n   as follows:\n   - `element_callback` is called on each `ElementEnd` signal with the \n     corresponding `ElementStart` tag and the result of the callback \n     invocation for the element&#39;s children.\n   - `data_callback` is called on each `Data` signal with the character data.  \n     This function won&#39;t be called twice consecutively or with the empty \n     string.\n - Other signals, returns an error.\n \n See [document_tree](#document_tree) for getting the entire document as a \n tree. \n\nSynonyms:\nxmlm.tree\nxmlm tree","url":"xmlm.html#tree"},{"doc":"xmlm","title":"with_encoding","content":"pub fn with_encoding(input: Input, encoding: Encoding) -&gt; Input\n `xmlm.with_encoding(input)` sets the `input` to use the given `encoding`.\n \nSynonyms:\nxmlm.with_encoding\nxmlm with_encoding","url":"xmlm.html#with_encoding"},{"doc":"xmlm","title":"with_entity_callback","content":"pub fn with_entity_callback(\n  input: Input,\n  entity_callback: fn(String) -&gt; Option(String),\n) -&gt; Input\n `xmlm.with_entity_callback(input, namespace_callback)` sets the `input` to \n use the given `entity_callback` to resolve non-predefined entity references.\n \n ## Example\n \n Imagine an XML document that looks something like this:\n \n xml\n &lt;p&gt; &amp;apple; &amp;pie; &lt;/p&gt;\n \n \n It has non-predifined entity references, and so when parsing, it will give \n an error.  To address this, we could use an entity callback function to \n resolve these references.\n \n \n xmlm.from_string(xml_data)\n |&gt; xmlm.with_entity_callback(fn(entity_reference) {\n   case entity_reference {\n     &quot;apple&quot; -&gt; Some(&quot;APPLE!&quot;)\n     &quot;pie&quot; -&gt; Some(&quot;PIE!&quot;)\n     _ -&gt; None\n   }\n })\n \n \n With that entity callback, the parsed `Data` signal would look something \n like this:\n \n \n Data(&quot;APPLE! PIE!&quot;)\n \n \nSynonyms:\nxmlm.with_entity_callback\nxmlm with_entity_callback","url":"xmlm.html#with_entity_callback"},{"doc":"xmlm","title":"with_namespace_callback","content":"pub fn with_namespace_callback(\n  input: Input,\n  namespace_callback: fn(String) -&gt; Option(String),\n) -&gt; Input\n `xmlm.with_namespace_callback(input, namespace_callback)` sets the `input` \n to use the given `namespace_callback` to bind undeclared namespace prefixes.\n \n ## Example\n \n Imagine an XML document something like this, that specifies a namespace.\n \n xml\n &lt;a xmlns:snazzy=&quot;https://www.example.com/snazzy&quot;&gt;\n   &lt;snazzy:b /&gt;\n &lt;/a&gt;\n \n \n This will parse Ok because the namespace is properly declared.  \n \n However, the following XML document would give an error, telling you about \n the unknown namespace prefix `snazzy`.\n \n xml\n &lt;a&gt;\n   &lt;snazzy:b /&gt;\n &lt;/a&gt;\n \n \n To address this, you may provide a function to bind undeclared namespace \n prefixes.\n \n \n xmlm.from_string(xml_data)\n |&gt; xmlm.with_namespace_callback(fn(prefix) {\n   case prefix {\n     &quot;snazzy&quot; -&gt; Some(&quot;https://www.example.com/snazzy&quot;)\n     _ -&gt; None\n   }\n })\n \n \n In this way, the `snazzy` prefix will be bound and no error will occur.\n \nSynonyms:\nxmlm.with_namespace_callback\nxmlm with_namespace_callback","url":"xmlm.html#with_namespace_callback"},{"doc":"xmlm","title":"with_stripping","content":"pub fn with_stripping(input: Input, stripping: Bool) -&gt; Input\n `xmlm.with_stripping(input, stripping)` sets the `input` to use the given\n `stripping`.\n \nSynonyms:\nxmlm.with_stripping\nxmlm with_stripping","url":"xmlm.html#with_stripping"},{"doc":"xmlm","title":"xmlm","content":" For usage examples and strategies for working with this library, check out \n the programs located in the `test/examples` directory.\n \n ## Notes\n \n - Don&#39;t forget to work with the `Input` that is returned by any \n   &quot;inputting&quot; function rather than the original.\n - If something is marked as being &quot;unspecified&quot;, do not depend on it.  \n   It may change at any time without a major version bump.  This mainly \n   applies to the various `*_to_string` functions.\n","url":"xmlm.html"}]);